/* 
 * leases.c -- tools to manage DHCP leases 
 * Russ Dill <Russ.Dill@asu.edu> July 2001
 */

#include <time.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "debug.h"
#include "dhcpd.h"
#include "files.h"
#include "options.h"
#include "leases.h"
#include "arpping.h"

//storm
#include "libgigarouter.h"
#include "gigaconfig.h"


unsigned char blank_chaddr[] = {[0 ... 15] = 0};

/* clear every lease out that chaddr OR yiaddr matches and is nonzero */
void clear_lease(u_int8_t *chaddr, u_int32_t yiaddr)
{
	unsigned int i, j;
	
	for (j = 0; j < 16 && !chaddr[j]; j++);
	
	for (i = 0; i < server_config.max_leases; i++)
		if ((j != 16 && !memcmp(leases[i].chaddr, chaddr, 16)) ||
		    (yiaddr && leases[i].yiaddr == yiaddr)) {
		    if (leases[i].static_f != 1)
		    {
			memset(&(leases[i]), 0, sizeof(struct dhcpOfferedAddr));
		    }
		}
}


/* add a lease into the table, clearing out any old ones */
struct dhcpOfferedAddr *add_lease(u_int8_t *chaddr, u_int32_t yiaddr, unsigned long lease)
{
	unsigned int i, j;
	struct dhcpOfferedAddr *oldest;

	//if static, update ip
	for (j = 0; j < 16 && !chaddr[j]; j++);

	for (i = 0; i < server_config.max_leases; i++)
	{
		if ((j != 16 && !memcmp(leases[i].chaddr, chaddr, 16)) ||
		    (yiaddr && leases[i].yiaddr == yiaddr)) {
		    if (leases[i].static_f == 1)
		    {
			leases[i].expires = time(0) + lease;
			return &leases[i];
			}
		}
	}
	
	/* clean out any old ones */
	clear_lease(chaddr, yiaddr);
		
	oldest = oldest_expired_lease();
	
	if (oldest) {
		memcpy(oldest->chaddr, chaddr, 16);
		oldest->yiaddr = yiaddr;
		oldest->expires = time(0) + lease;
	}
	
	return oldest;
}

/* add a  static lease into the table */
struct dhcpOfferedAddr *add_static_lease(u_int8_t *chaddr, u_int32_t yiaddr, unsigned long lease)
{
	struct dhcpOfferedAddr *oldest;
	
	/* clean out any old ones */
	clear_lease(chaddr, yiaddr);

	oldest = oldest_expired_lease();
	
	if (oldest) {
//        printf("====debug add chaddr %x:%x:%x:%x:%x:%x ===\n",chaddr[0],chaddr[1],chaddr[2],chaddr[3],chaddr[4],chaddr[5]);
		memcpy(oldest->chaddr, chaddr, 16);
//        printf("====debug after add chaddr %x:%x:%x:%x:%x:%x ===\n",oldest->chaddr[0],oldest->chaddr[1],oldest->chaddr[2],oldest->chaddr[3],oldest->chaddr[4],oldest->chaddr[5]);
		oldest->yiaddr = yiaddr;
		oldest->expires = lease;
		oldest->static_f = 1;
	}
	
	return oldest;
}

/* true if a lease has expired */
int lease_expired(struct dhcpOfferedAddr *lease)
{
	return (lease->expires < (unsigned long) time(0));
}	


/* Find the oldest expired lease, NULL if there are no expired leases */
struct dhcpOfferedAddr *oldest_expired_lease(void)
{
	struct dhcpOfferedAddr *oldest = NULL;
	unsigned long oldest_lease = time(0);
	unsigned int i;

	
	for (i = 0; i < server_config.max_leases; i++)
		if (oldest_lease > leases[i].expires) {
			//if static, skip it
			if (leases[i].static_f != 1)
			{
				oldest_lease = leases[i].expires;
				oldest = &(leases[i]);
			}
		}
	return oldest;
		
}


/* Find the first lease that matches chaddr, NULL if no match */
struct dhcpOfferedAddr *find_lease_by_chaddr(u_int8_t *chaddr)
{
	unsigned int i;
	for (i = 0; i < server_config.max_leases; i++)
	{
{
        struct in_addr addr;
        u_int8_t* addr1;

        addr.s_addr = leases[i].yiaddr;
//        printf("====debug %d: yiaddr %s ===\n",i,inet_ntoa(addr));
//        printf("====debug %d: chaddr %x:%x:%x:%x:%x:%x ===\n",i,chaddr[0],chaddr[1],chaddr[2],chaddr[3],chaddr[4],chaddr[5]);
        addr1 = leases[i].chaddr;
//        printf("====debug %d: leases.chaddr %x:%x:%x:%x:%x:%x ===\n",i,addr1[0],addr1[1],addr1[2],addr1[3],addr1[4],addr1[5]);
}
		if (!memcmp(leases[i].chaddr, chaddr, 16)) 
                       return &(leases[i]);
	}
	return NULL;
}


/* Find the first lease that matches yiaddr, NULL is no match */
struct dhcpOfferedAddr *find_lease_by_yiaddr(u_int32_t yiaddr)
{
	unsigned int i;

	for (i = 0; i < server_config.max_leases; i++)
		if (leases[i].yiaddr == yiaddr) return &(leases[i]);
	
	return NULL;
}


/* find an assignable address, it check_expired is true, we check all the expired leases as well.
 * Maybe this should try expired leases by age... */
u_int32_t find_address(int check_expired) 
{
	u_int32_t addr, ret;
	struct dhcpOfferedAddr *lease = NULL;		

	addr = ntohl(server_config.start); /* addr is in host order here */
	for (;addr <= ntohl(server_config.end); addr++) {

		/* ie, 192.168.55.0 */
//		if (!(addr & 0xFF)) continue;

		/* ie, 192.168.55.255 */
//		if ((addr & 0xFF) == 0xFF) continue;

		/* lease is not taken */
		ret = htonl(addr);
		if ((!(lease = find_lease_by_yiaddr(ret)) ||

		     /* or it expired and we are checking for expired leases */
		     (check_expired  && 
		     (lease_expired(lease) && lease->static_f != 1))) &&

		     /* and it isn't on the network */
	    	     !check_ip(ret) && !check_host_ip(ret) ) {
			return ret;
			break;
		}
	}
	return 0;
}


/* check is an IP is taken, if it is, add it to the lease table */
int check_ip(u_int32_t addr)
{

	struct in_addr temp;
	
	if (arpping(addr, server_config.server, server_config.arp, server_config.interface) == 0) {
		temp.s_addr = addr;
	 	LOG(LOG_INFO, "%s belongs to someone, reserving it for %ld seconds", 
	 		inet_ntoa(temp), server_config.conflict_time);
		//not to add blank_chaddr lease
		//add_lease(blank_chaddr, addr, server_config.conflict_time);
		return 1;
	} else return 0;
}

/* check is an IP is taken, if it is, add it to the lease table */
int check_host_ip(u_int32_t addr)
{
	unsigned char blank_chaddr[] = {[0 ... 15] = 0};
	struct in_addr temp;
	int nResult=0,nDebug=0;
	char pppBuffer[16]="";
	
     nResult = GIGA_GetSystemSetting("LAN", "lan_ipaddr1" , pppBuffer, 16 , RW_ITEM_MODE );

    if(nDebug)
        LOG(LOG_INFO,"udhcpd: Get system setting lan_ipaddr1:%s nResult=%d\n",pppBuffer, nResult);

    if ( nResult != 0 )
    {
       LOG(LOG_INFO,"udhcpd: Get system setting lan_ipaddr1:%s fail (nResult=%d)\n",pppBuffer, nResult);
        return 0;
    }	
    
    if(nDebug){
	temp.s_addr = addr;
       LOG(LOG_INFO,"check ip:%s\n",inet_ntoa(temp));
    }
       
	if ( inet_addr(pppBuffer) == addr ) {
		temp.s_addr = addr;
	 	LOG(LOG_INFO, "%s belongs to Host, don't leases it!", pppBuffer);
		//not to add blank_chaddr lease
		//add_lease(blank_chaddr, addr, server_config.conflict_time);
		return 1;
	} else return 0;
}
